# Generated by roxygen2: do not edit by hand

S3method(afex_plot,afex_aov)
S3method(afex_plot,default)
S3method(afex_plot,merMod)
S3method(afex_plot,mixed)
S3method(anova,afex_aov)
S3method(anova,mixed)
S3method(fitted,afex_aov)
S3method(nice,afex_aov)
S3method(nice,anova)
S3method(nice,mixed)
S3method(predict,afex_aov)
S3method(print,afex_aov)
S3method(print,mixed)
S3method(print,nice_table)
S3method(residuals,afex_aov)
S3method(summary,afex_aov)
S3method(summary,mixed)
export(afex_options)
export(afex_plot)
export(aov_4)
export(aov_car)
export(aov_ez)
export(compare.2.vectors)
export(ems)
export(interaction_plot)
export(lmer)
export(lmer_alt)
export(mixed)
export(nice)
export(oneway_plot)
export(round_ps)
export(round_ps_apa)
export(set_default_contrasts)
export(set_deviation_contrasts)
export(set_effects_contrasts)
export(set_sum_contrasts)
export(set_treatment_contrasts)
export(test_levene)
export(test_sphericity)
importFrom(car,Anova)
importFrom(lme4,findbars)
importFrom(lme4,getME)
importFrom(lme4,glmer)
importFrom(lme4,isREML)
importFrom(lme4,nobars)
importFrom(lmerTest,lmer)
importFrom(methods,as)
importFrom(methods,is)
importFrom(parallel,clusterApplyLB)
importFrom(parallel,clusterCall)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(reshape2,dcast)
importFrom(stats,"contrasts<-")
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,qt)
importFrom(stats,reshape)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,symnum)
importFrom(stats,t.test)
importFrom(stats,terms)
importFrom(stats,wilcox.test)
importFrom(stats,xtabs)
importFrom(utils,head)
importFrom(utils,packageVersion)
